/*
     Copyrights Bracket Software. All rights reserved.
     Sole Proprietor: Sidharth Yadav

     The code comes without any guarantee and you use it only at your own risk. Bracket Software or its sole proprietor Sidharth Yadav cannot be held liable for any 
     damage of any kind arising out of using this code directly or indirectly.
*/

package bracket.webservices.samples.test;

import java.awt.Component;

import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;

import java.net.HttpURLConnection;

import bracket.webservices.client.Service;
import bracket.webservices.client.ServiceRequest;
import bracket.webservices.client.ServiceResponse;
import bracket.webservices.client.ServiceStateListener;

/**
     This is just a test service. It does nothing other than sending a plain html message about itself.
     @author Sidharth
 */
public final class TestService implements Service
{
     private String[] classpathEntries = new String[0];
     private String name="testservice";
     private int status=STOPPED;
     private int start_mode=START_MODE_AUTOMATIC;

     private Map service_headers=null;
     private LinkedList service_state_listeners=null;

     public TestService()
     {
     }

     public String getId()
     {
          //This should return the fully qualified name of this service class.
          return TestService.class.getName();
     }

     public String getName()
     {
          return name;
     }

     public String getDescription()
     {
          return "It is just a test service. It does nothing.";
     }

     public int getStatus()
     {
          return status;
     }

     public String getLastStatusString()
     {
          return null;
     }

     public void setStartMode(int mode)
     {
          if (mode == START_MODE_AUTOMATIC ||mode == START_MODE_MANUAL)
               this.start_mode=mode;
     }

     public int getStartMode()
     {
          return start_mode;
     }

     public String getVersion()
     {
          return "1.0";
     }

     public void init(Map session_context)
     {
          //The session_context map can be used to obtain values related to Bracket session.
          //Key "session.home" returns the home folder of the current Bracket client session, key "session.owner" returns the username of Bracket client session owner.
          //However this test service doesn't need those values so we would not use them.

          service_headers=new HashMap();
          service_headers.put("BracketServiceName", getName());
          service_headers.put("BracketServiceVersion", getVersion());

          service_state_listeners=new LinkedList();

          status=STOPPED;
     }

     public boolean start()
     {
          if (status==RUNNING)
               return true;

          status=RUNNING;
          fireServiceStateEvent(STOPPED, RUNNING);
          return true;
     }

     public boolean stop()
     {
          if (status==STOPPED)
               return true;

          status=STOPPED;
          fireServiceStateEvent(RUNNING, STOPPED);
          return true;
     }

     public void doServe(ServiceRequest request, ServiceResponse response) throws Exception
     {
          //Nothing much to do for this test service. For other services do the processing required inside this method.
          //This method is similar to doGet and doPut methods of any servlet implementation.

          //Use request.getParameter(String) method to be value of any request parameters.
          //Use response.setParameter(String, String) to set any response parameters.

          //Write the content using response.getResponseWriter().write(byte[], int, int) method.

          //Session management can be done using request.getSession(boolean) method. It's functioning is similar to the getSession(boolean) method of javax.servlet.http.HttpServletRequest interface.

          byte[] html_message_bytes=("<html><head><title>Test Service</title></head><body><h3>Service <font color=magenta>"+getName()+"</font> is just a test service. It is running successfully. <br>However it does nothing.</h3></body></html>").getBytes();

          response.setResponseCode(HttpURLConnection.HTTP_OK);
          response.setResponseMessage("OK");
          response.setParameter("Content-length", ""+html_message_bytes.length);
          response.setParameter("Content-type", "text/html");

          response.getResponseWriter().write(html_message_bytes, 0, html_message_bytes.length);
     }

     public Map getServiceHeaders()
     {
          return service_headers;
     }

     public Component getGUI()
     {
          //We don't have any service attributes to be managed for this service inside Webservices Control Panel of Bracket client software.
          //You may however return a gui that manages state/attributes of your service impl.

          return null;
     }

     public void setClasspathEntries(String[] classpathEntries)
     {
          if (classpathEntries!=null)
               this.classpathEntries = classpathEntries;
          else
               this.classpathEntries=new String[0];
     }

     public String[] getClasspathEntries()
     {
          return classpathEntries;
     }

     public void addServiceStateListener(ServiceStateListener ssl)
     {
          if (ssl!=null)
               service_state_listeners.add(ssl);
     }

     public void removeServiceStateListener(ServiceStateListener ssl)
     {
          if (ssl!=null)
               service_state_listeners.remove(ssl);
     }

     private void fireServiceStateEvent(int old_status, int new_status)
     {
          for (int i=0; i<service_state_listeners.size(); i++)
          {
               ((ServiceStateListener) service_state_listeners.get(i)).serviceStateChanged(this, old_status, new_status);
          }
     }

     public boolean dispose()
     {
          stop();
          classpathEntries=null;
          service_state_listeners=null;
          service_headers=null;
          status=DISPOSED;

          return true;
     }
}